from enum import Enum
from typing import Type, Union


from .models import (
    EthereumLabel,
    SepoliaLabel,
    PolygonLabel,
    MumbaiLabel,
    AmoyLabel,
    XDaiLabel,
    ZkSyncEraLabel,
    ZkSyncEraSepoliaLabel,
    BaseLabel,
    ArbitrumNovaLabel,
    ArbitrumOneLabel,
    ArbitrumSepoliaLabel,
    Game7OrbitArbitrumSepoliaLabel,
    XaiLabel,
    XaiSepoliaLabel,
    AvalancheLabel,
    AvalancheFujiLabel,
    BlastLabel,
    BlastSepoliaLabel,
    ProofOfPlayApexLabel,
    StarknetLabel,
    StarknetSepoliaLabel,
    MantleLabel,
    MantleSepoliaLabel,
)


class AvailableBlockchainType(Enum):
    ETHEREUM = "ethereum"
    POLYGON = "polygon"
    MUMBAI = "mumbai"
    AMOY = "amoy"
    XDAI = "xdai"
    SEPOLIA = "sepolia"
    ZKSYNC_ERA = "zksync_era"
    ZKSYNC_ERA_TESTNET = "zksync_era_testnet"
    ZKSYNC_ERA_SEPOLIA = "zksync_era_sepolia"
    ARBITRUM_ONE = "arbitrum_one"
    ARBITRUM_NOVA = "arbitrum_nova"
    ARBITRUM_SEPOLIA = "arbitrum_sepolia"
    XAI = "xai"
    XAI_SEPOLIA = "xai_sepolia"
    AVALANCHE = "avalanche"
    AVALANCHE_FUJI = "avalanche_fuji"
    BLAST = "blast"
    BLAST_SEPOLIA = "blast_sepolia"
    PROOFOFPLAY_APEX = "proofofplay_apex"
    STARKNET = "starknet"
    STARKNET_SEPOLIA = "starknet_sepolia"
    MANTLE = "mantle"
    MANTLE_SEPOLIA = "mantle_sepolia"
    GAME7_ORBIT_ARBITRUM_SEPOLIA = "game7_orbit_arbitrum_sepolia"


def get_label_model(blockchain_type: AvailableBlockchainType) -> Type[
    Union[
        EthereumLabel,
        SepoliaLabel,
        PolygonLabel,
        MumbaiLabel,
        AmoyLabel,
        XDaiLabel,
        ZkSyncEraLabel,
        ZkSyncEraSepoliaLabel,
        BaseLabel,
        ArbitrumNovaLabel,
        ArbitrumOneLabel,
        ArbitrumSepoliaLabel,
        Game7OrbitArbitrumSepoliaLabel,
        XaiLabel,
        XaiSepoliaLabel,
        AvalancheLabel,
        AvalancheFujiLabel,
        BlastLabel,
        BlastSepoliaLabel,
        ProofOfPlayApexLabel,
        StarknetLabel,
        StarknetSepoliaLabel,
        MantleLabel,
        MantleSepoliaLabel,
    ]
]:
    """
    Depends on provided blockchain type set proper blocks model.
    """

    label_model: Type[
        Union[
            EthereumLabel,
            SepoliaLabel,
            PolygonLabel,
            MumbaiLabel,
            AmoyLabel,
            XDaiLabel,
            ZkSyncEraLabel,
            ZkSyncEraSepoliaLabel,
            BaseLabel,
            ArbitrumNovaLabel,
            ArbitrumOneLabel,
            ArbitrumSepoliaLabel,
            Game7OrbitArbitrumSepoliaLabel,
            XaiLabel,
            XaiSepoliaLabel,
            AvalancheLabel,
            AvalancheFujiLabel,
            BlastLabel,
            BlastSepoliaLabel,
            ProofOfPlayApexLabel,
            StarknetLabel,
            StarknetSepoliaLabel,
            MantleLabel,
            MantleSepoliaLabel,
        ]
    ]

    if blockchain_type == AvailableBlockchainType.ETHEREUM:
        label_model = EthereumLabel
    elif blockchain_type == AvailableBlockchainType.SEPOLIA:
        label_model = SepoliaLabel
    elif blockchain_type == AvailableBlockchainType.POLYGON:
        label_model = PolygonLabel
    elif blockchain_type == AvailableBlockchainType.MUMBAI:
        label_model = MumbaiLabel
    elif blockchain_type == AvailableBlockchainType.AMOY:
        label_model = AmoyLabel
    elif blockchain_type == AvailableBlockchainType.XDAI:
        label_model = XDaiLabel
    elif blockchain_type == AvailableBlockchainType.ZKSYNC_ERA:
        label_model = ZkSyncEraLabel
    elif blockchain_type == AvailableBlockchainType.ZKSYNC_ERA_SEPOLIA:
        label_model = ZkSyncEraSepoliaLabel
    elif blockchain_type == AvailableBlockchainType.ARBITRUM_ONE:
        label_model = ArbitrumOneLabel
    elif blockchain_type == AvailableBlockchainType.ARBITRUM_NOVA:
        label_model = ArbitrumNovaLabel
    elif blockchain_type == AvailableBlockchainType.ARBITRUM_SEPOLIA:
        label_model = ArbitrumSepoliaLabel
    elif blockchain_type == AvailableBlockchainType.XAI:
        label_model = XaiLabel
    elif blockchain_type == AvailableBlockchainType.XAI_SEPOLIA:
        label_model = XaiSepoliaLabel
    elif blockchain_type == AvailableBlockchainType.AVALANCHE:
        label_model = AvalancheLabel
    elif blockchain_type == AvailableBlockchainType.AVALANCHE_FUJI:
        label_model = AvalancheFujiLabel
    elif blockchain_type == AvailableBlockchainType.BLAST:
        label_model = BlastLabel
    elif blockchain_type == AvailableBlockchainType.BLAST_SEPOLIA:
        label_model = BlastSepoliaLabel
    elif blockchain_type == AvailableBlockchainType.PROOFOFPLAY_APEX:
        label_model = ProofOfPlayApexLabel
    elif blockchain_type == AvailableBlockchainType.STARKNET:
        label_model = StarknetLabel
    elif blockchain_type == AvailableBlockchainType.STARKNET_SEPOLIA:
        label_model = StarknetSepoliaLabel
    elif blockchain_type == AvailableBlockchainType.MANTLE:
        label_model = MantleLabel
    elif blockchain_type == AvailableBlockchainType.MANTLE_SEPOLIA:
        label_model = MantleSepoliaLabel
    elif blockchain_type == AvailableBlockchainType.GAME7_ORBIT_ARBITRUM_SEPOLIA:
        label_model = Game7OrbitArbitrumSepoliaLabel
    else:
        raise ValueError(f"Unknown blockchain type: {blockchain_type}")

    return label_model



from setuptools import find_packages, setup

from moonstreamdbv3.version import VERSION

long_description = ""
with open("README.md") as ifp:
    long_description = ifp.read()

setup(
    name="moonstreamdb-v3",
    version=VERSION,
    author="Bugout.dev",
    author_email="engineers@bugout.dev",
    license="Apache License 2.0",
    description="Moonstream V3 database",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bugout-dev/moonstream",
    platforms="all",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8",
    packages=find_packages(),
    package_data={"moonstreamdbv3": ["py.typed"]},
    zip_safe=False,
    install_requires=["alembic", "psycopg2-binary", "sqlalchemy>=2.0.4"],
    extras_require={
        "dev": ["black", "isort", "mypy"],
        "distribute": ["setuptools", "twine", "wheel"],
    },
)



from logging.config import fileConfig

from alembic import context
from sqlalchemy import engine_from_config, pool

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
if config.config_file_name is not None:
    fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
from moonstreamdbv3.models_indexes import Base as MoonstreamBase

target_metadata = MoonstreamBase.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.

from moonstreamdbv3.models_indexes import (
    AbiJobs,
    AbiSubscriptions,
    ArbitrumOneBlockIndex,
    ArbitrumOneLogIndex,
    ArbitrumOneReorgs,
    ArbitrumOneTransactionIndex,
    ArbitrumSepoliaBlockIndex,
    ArbitrumSepoliaLogIndex,
    ArbitrumSepoliaReorgs,
    ArbitrumSepoliaTransactionIndex,
    EthereumBlockIndex,
    EthereumLogIndex,
    EthereumReorgs,
    EthereumTransactionIndex,
    Game7OrbitArbitrumSepoliaBlockIndex,
    Game7OrbitArbitrumSepoliaLogIndex,
    Game7OrbitArbitrumSepoliaReorgs,
    Game7OrbitArbitrumSepoliaTransactionIndex,
    ImxZkevmBlockIndex,
    ImxZkevmLogIndex,
    ImxZkevmReorgs,
    ImxZkevmSepoliaBlockIndex,
    ImxZkevmSepoliaLogIndex,
    ImxZkevmSepoliaReorgs,
    ImxZkevmSepoliaTransactionIndex,
    ImxZkevmTransactionIndex,
    MantleBlockIndex,
    MantleLogIndex,
    MantleReorgs,
    MantleSepoliaBlockIndex,
    MantleSepoliaLogIndex,
    MantleSepoliaReorgs,
    MantleSepoliaTransactionIndex,
    MantleTransactionIndex,
    PolygonBlockIndex,
    PolygonLogIndex,
    PolygonReorgs,
    PolygonTransactionIndex,
    XaiBlockIndex,
    XaiLogIndex,
    XaiReorgs,
    XaiSepoliaBlockIndex,
    XaiSepoliaLogIndex,
    XaiSepoliaReorgs,
    XaiSepoliaTransactionIndex,
    XaiTransactionIndex,
)


def include_symbol(tablename, schema):
    return tablename in {
        EthereumBlockIndex.__tablename__,
        EthereumTransactionIndex.__tablename__,
        EthereumLogIndex.__tablename__,
        EthereumReorgs.__tablename__,
        PolygonBlockIndex.__tablename__,
        PolygonTransactionIndex.__tablename__,
        PolygonLogIndex.__tablename__,
        PolygonReorgs.__tablename__,
        XaiBlockIndex.__tablename__,
        XaiTransactionIndex.__tablename__,
        XaiLogIndex.__tablename__,
        XaiReorgs.__tablename__,
        XaiSepoliaBlockIndex.__tablename__,
        XaiSepoliaTransactionIndex.__tablename__,
        XaiSepoliaLogIndex.__tablename__,
        XaiSepoliaReorgs.__tablename__,
        ArbitrumOneBlockIndex.__tablename__,
        ArbitrumOneTransactionIndex.__tablename__,
        ArbitrumOneLogIndex.__tablename__,
        ArbitrumOneReorgs.__tablename__,
        ArbitrumSepoliaBlockIndex.__tablename__,
        ArbitrumSepoliaTransactionIndex.__tablename__,
        ArbitrumSepoliaLogIndex.__tablename__,
        ArbitrumSepoliaReorgs.__tablename__,
        Game7OrbitArbitrumSepoliaBlockIndex.__tablename__,
        Game7OrbitArbitrumSepoliaTransactionIndex.__tablename__,
        Game7OrbitArbitrumSepoliaLogIndex.__tablename__,
        Game7OrbitArbitrumSepoliaReorgs.__tablename__,
        MantleBlockIndex.__tablename__,
        MantleTransactionIndex.__tablename__,
        MantleLogIndex.__tablename__,
        MantleReorgs.__tablename__,
        MantleSepoliaBlockIndex.__tablename__,
        MantleSepoliaTransactionIndex.__tablename__,
        MantleSepoliaLogIndex.__tablename__,
        MantleSepoliaReorgs.__tablename__,
        AbiJobs.__tablename__,
        AbiSubscriptions.__tablename__,
        ImxZkevmBlockIndex.__tablename__,
        ImxZkevmTransactionIndex.__tablename__,
        ImxZkevmLogIndex.__tablename__,
        ImxZkevmReorgs.__tablename__,
        ImxZkevmSepoliaBlockIndex.__tablename__,
        ImxZkevmSepoliaTransactionIndex.__tablename__,
        ImxZkevmSepoliaLogIndex.__tablename__,
        ImxZkevmSepoliaReorgs.__tablename__,
    }


def run_migrations_offline() -> None:
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = config.get_main_option("sqlalchemy.url")
    context.configure(
        url=url,
        target_metadata=target_metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
        version_table="alembic_version",
        include_schemas=True,
        include_symbol=include_symbol,
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online() -> None:
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    connectable = engine_from_config(
        config.get_section(config.config_ini_section, {}),
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            version_table="alembic_version",
            include_schemas=True,
            include_symbol=include_symbol,
        )

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



import uuid

from sqlalchemy import (
    VARCHAR,
    BigInteger,
    Boolean,
    Column,
    DateTime,
    ForeignKey,
    Index,
    Integer,
    LargeBinary,
    MetaData,
    PrimaryKeyConstraint,
    Text,
    UniqueConstraint,
)
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql import expression

"""
Naming conventions doc
https://docs.sqlalchemy.org/en/13/core/constraints.html#configuring-constraint-naming-conventions
"""
convention = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(constraint_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}
metadata = MetaData(naming_convention=convention)
Base = declarative_base(metadata=metadata)


class utcnow(expression.FunctionElement):
    type = DateTime  # type: ignore


@compiles(utcnow, "postgresql")
def pg_utcnow(element, compiler, **kwargs):
    return "TIMEZONE('utc', statement_timestamp())"


class EvmBasedBlocks(Base):
    __abstract__ = True

    block_number = Column(BigInteger, primary_key=True, nullable=False, index=True)
    block_hash = Column(VARCHAR(256), nullable=False, index=False)
    block_timestamp = Column(BigInteger, nullable=False, index=True)
    parent_hash = Column(VARCHAR(256), nullable=False)
    row_id = Column(BigInteger, nullable=False, index=False)
    path = Column(Text, nullable=False)
    indexed_at = Column(
        DateTime(timezone=True), server_default=utcnow(), nullable=False
    )


class EvmBasedTransactions(Base):
    __abstract__ = True

    hash = Column(
        VARCHAR(256), primary_key=True, unique=True, nullable=False, index=True
    )
    from_address = Column(LargeBinary(length=20), nullable=False, index=True)
    to_address = Column(LargeBinary(length=20), nullable=False, index=True)
    selector = Column(VARCHAR(256), nullable=True, index=True)
    type = Column(Integer, nullable=True, index=True)
    row_id = Column(BigInteger, nullable=False, index=False)
    block_hash = Column(VARCHAR(256), nullable=False, index=True)
    index = Column(BigInteger, nullable=False, index=True)
    path = Column(Text, nullable=False)
    indexed_at = Column(
        DateTime(timezone=True), server_default=utcnow(), nullable=False
    )


class EvmBasedLogs(Base):
    __abstract__ = True

    block_hash = Column(VARCHAR(256), nullable=False, index=True)
    address = Column(LargeBinary(length=20), nullable=False, index=True)
    row_id = Column(BigInteger, nullable=False, index=False)
    selector = Column(VARCHAR(256), nullable=True, index=False)
    topic1 = Column(VARCHAR(256), nullable=True, index=False)
    topic2 = Column(VARCHAR(256), nullable=True, index=False)
    topic3 = Column(VARCHAR(256), nullable=True, index=False)
    log_index = Column(BigInteger, nullable=False, index=False)
    path = Column(Text, nullable=False)
    indexed_at = Column(
        DateTime(timezone=True), server_default=utcnow(), nullable=False
    )


class EvmBasedReorgs(Base):
    __abstract__ = True
    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    block_number = Column(BigInteger, nullable=False, index=True)
    block_hash = Column(VARCHAR(256), nullable=False, index=True)


### Ethereum


class EthereumBlockIndex(EvmBasedBlocks):
    __tablename__ = "ethereum_blocks"


class EthereumTransactionIndex(EvmBasedTransactions):
    __tablename__ = "ethereum_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("ethereum_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class EthereumLogIndex(EvmBasedLogs):
    __tablename__ = "ethereum_logs"

    __table_args__ = (
        Index(
            "idx_ethereum_logs_address_selector", "address", "selector", unique=False
        ),
        Index(
            "idx_ethereum_logs_block_hash_log_index",
            "block_hash",
            "log_index",
            unique=True,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_ethereum_log_index_transaction_hash_log_index",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_ethereum_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("ethereum_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class EthereumReorgs(EvmBasedReorgs):
    __tablename__ = "ethereum_reorgs"


### Sepolia


class SepoliaBlockIndex(EvmBasedBlocks):
    __tablename__ = "sepolia_blocks"


class SepoliaTransactionIndex(EvmBasedTransactions):
    __tablename__ = "sepolia_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("sepolia_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class SepoliaLogIndex(EvmBasedLogs):

    __tablename__ = "sepolia_logs"

    __table_args__ = (
        Index(
            "idx_sepolia_logs_address_selector",
            "address",
            "selector",
            unique=False,
        ),
        Index(
            "idx_sepolia_logs_block_hash_log_index",
            "block_hash",
            "log_index",
            unique=True,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_sepolia_log_index_transaction_hash_log_index",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_sepolia_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("sepolia_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class SepoliaReorgs(EvmBasedReorgs):
    __tablename__ = "sepolia_reorgs"


### Polygon


class PolygonBlockIndex(EvmBasedBlocks):
    __tablename__ = "polygon_blocks"


class PolygonTransactionIndex(EvmBasedTransactions):
    __tablename__ = "polygon_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("polygon_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class PolygonLogIndex(EvmBasedLogs):

    __tablename__ = "polygon_logs"

    __table_args__ = (
        Index("idx_polygon_logs_address_selector", "address", "selector", unique=False),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_polygon_log_index_transaction_hash_log_index",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_polygon_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("polygon_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class PolygonReorgs(EvmBasedReorgs):
    __tablename__ = "polygon_reorgs"


### Xai


class XaiBlockIndex(EvmBasedBlocks):
    __tablename__ = "xai_blocks"

    l1_block_number = Column(BigInteger, nullable=False)


class XaiTransactionIndex(EvmBasedTransactions):
    __tablename__ = "xai_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("xai_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class XaiLogIndex(EvmBasedLogs):

    __tablename__ = "xai_logs"

    __table_args__ = (
        Index("idx_xai_logs_address_selector", "address", "selector", unique=False),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_xai_log_idx_tx_hash_log_idx",
        ),
        PrimaryKeyConstraint("transaction_hash", "log_index", name="pk_xai_log_index"),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("xai_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class XaiReorgs(EvmBasedReorgs):
    __tablename__ = "xai_reorgs"


### Xai Sepolia


class XaiSepoliaBlockIndex(EvmBasedBlocks):
    __tablename__ = "xai_sepolia_blocks"

    l1_block_number = Column(BigInteger, nullable=False)


class XaiSepoliaTransactionIndex(EvmBasedTransactions):
    __tablename__ = "xai_sepolia_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("xai_sepolia_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class XaiSepoliaLogIndex(EvmBasedLogs):

    __tablename__ = "xai_sepolia_logs"

    __table_args__ = (
        Index(
            "idx_xai_sepolia_logs_address_selector", "address", "selector", unique=False
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_xai_sepolia_log_idx_tx_hash_log_idx",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_xai_sepolia_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("xai_sepolia_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class XaiSepoliaReorgs(EvmBasedReorgs):
    __tablename__ = "xai_sepolia_reorgs"


### Arbitrum One


class ArbitrumOneBlockIndex(EvmBasedBlocks):
    __tablename__ = "arbitrum_one_blocks"

    l1_block_number = Column(BigInteger, nullable=False)


class ArbitrumOneTransactionIndex(EvmBasedTransactions):
    __tablename__ = "arbitrum_one_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("arbitrum_one_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class ArbitrumOneLogIndex(EvmBasedLogs):

    __tablename__ = "arbitrum_one_logs"

    __table_args__ = (
        Index(
            "idx_arbitrum_one_logs_address_selector",
            "address",
            "selector",
            unique=False,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_arbitrum_one_log_idx_tx_hash_log_idx",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_arbitrum_one_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("arbitrum_one_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class ArbitrumOneReorgs(EvmBasedReorgs):
    __tablename__ = "arbitrum_one_reorgs"


### Arbitrum Sepolia


class ArbitrumSepoliaBlockIndex(EvmBasedBlocks):
    __tablename__ = "arbitrum_sepolia_blocks"

    l1_block_number = Column(BigInteger, nullable=False)


class ArbitrumSepoliaTransactionIndex(EvmBasedTransactions):
    __tablename__ = "arbitrum_sepolia_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("arbitrum_sepolia_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class ArbitrumSepoliaLogIndex(EvmBasedLogs):

    __tablename__ = "arbitrum_sepolia_logs"

    __table_args__ = (
        Index(
            "idx_arbitrum_sepolia_logs_address_selector",
            "address",
            "selector",
            unique=False,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_arbitrum_sepolia_log_idx_tx_hash_log_idx",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_arbitrum_sepolia_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("arbitrum_sepolia_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class ArbitrumSepoliaReorgs(EvmBasedReorgs):
    __tablename__ = "arbitrum_sepolia_reorgs"


### Game7 Orbit Arbitrum Sepolia


class Game7OrbitArbitrumSepoliaBlockIndex(EvmBasedBlocks):
    __tablename__ = "game7_orbit_arbitrum_sepolia_blocks"

    l1_block_number = Column(BigInteger, nullable=False)


class Game7OrbitArbitrumSepoliaTransactionIndex(EvmBasedTransactions):
    __tablename__ = "game7_orbit_arbitrum_sepolia_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey(
            "game7_orbit_arbitrum_sepolia_blocks.block_number", ondelete="CASCADE"
        ),
        nullable=False,
        index=True,
    )


class Game7OrbitArbitrumSepoliaLogIndex(EvmBasedLogs):

    __tablename__ = "game7_orbit_arbitrum_sepolia_logs"

    __table_args__ = (
        Index(
            "idx_game7_orbit_arbitrum_sepolia_logs_address_selector",
            "address",
            "selector",
            unique=False,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_g7o_arbitrum_sepolia_log_idx_tx_hash_log_idx",
        ),
        PrimaryKeyConstraint(
            "transaction_hash",
            "log_index",
            name="pk_game7_orbit_arbitrum_sepolia_log_index",
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey(
            "game7_orbit_arbitrum_sepolia_transactions.hash", ondelete="CASCADE"
        ),
        nullable=False,
        index=True,
    )


class Game7OrbitArbitrumSepoliaReorgs(EvmBasedReorgs):
    __tablename__ = "game7_orbit_arbitrum_sepolia_reorgs"


### Mantle


class MantleBlockIndex(EvmBasedBlocks):
    __tablename__ = "mantle_blocks"


class MantleTransactionIndex(EvmBasedTransactions):
    __tablename__ = "mantle_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("mantle_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class MantleLogIndex(EvmBasedLogs):
    __tablename__ = "mantle_logs"

    __table_args__ = (
        Index("idx_mantle_logs_address_selector", "address", "selector", unique=False),
        Index(
            "idx_mantle_logs_block_hash_log_index",
            "block_hash",
            "log_index",
            unique=True,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_mantle_log_index_transaction_hash_log_index",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_mantle_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("mantle_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class MantleReorgs(EvmBasedReorgs):
    __tablename__ = "mantle_reorgs"


### Mantle Sepolia


class MantleSepoliaBlockIndex(EvmBasedBlocks):
    __tablename__ = "mantle_sepolia_blocks"


class MantleSepoliaTransactionIndex(EvmBasedTransactions):
    __tablename__ = "mantle_sepolia_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("mantle_sepolia_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class MantleSepoliaLogIndex(EvmBasedLogs):
    __tablename__ = "mantle_sepolia_logs"

    __table_args__ = (
        Index(
            "idx_mantle_sepolia_logs_address_selector",
            "address",
            "selector",
            unique=False,
        ),
        Index(
            "idx_mantle_sepolia_logs_block_hash_log_index",
            "block_hash",
            "log_index",
            unique=True,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_mantle_sepolia_log_index_transaction_hash_log_index",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_mantle_sepolia_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("mantle_sepolia_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class MantleSepoliaReorgs(EvmBasedReorgs):
    __tablename__ = "mantle_sepolia_reorgs"


### Immutable zkEvm


class ImxZkevmBlockIndex(EvmBasedBlocks):
    __tablename__ = "imx_zkevm_blocks"


class ImxZkevmTransactionIndex(EvmBasedTransactions):
    __tablename__ = "imx_zkevm_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("imx_zkevm_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class ImxZkevmLogIndex(EvmBasedLogs):
    __tablename__ = "imx_zkevm_logs"

    __table_args__ = (
        Index(
            "idx_imx_zkevm_logs_address_selector",
            "address",
            "selector",
            unique=False,
        ),
        Index(
            "idx_imx_zkevm_logs_block_hash_log_index",
            "block_hash",
            "log_index",
            unique=True,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_imx_zkevm_log_index_transaction_hash_log_index",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_imx_zkevm_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("imx_zkevm_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class ImxZkevmReorgs(EvmBasedReorgs):
    __tablename__ = "imx_zkevm_reorgs"


### Immutable zkEvm Sepolia


class ImxZkevmSepoliaBlockIndex(EvmBasedBlocks):
    __tablename__ = "imx_zkevm_sepolia_blocks"


class ImxZkevmSepoliaTransactionIndex(EvmBasedTransactions):
    __tablename__ = "imx_zkevm_sepolia_transactions"

    block_number = Column(
        BigInteger,
        ForeignKey("imx_zkevm_sepolia_blocks.block_number", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class ImxZkevmSepoliaLogIndex(EvmBasedLogs):
    __tablename__ = "imx_zkevm_sepolia_logs"

    __table_args__ = (
        Index(
            "idx_imx_zkevm_sepolia_logs_address_selector",
            "address",
            "selector",
            unique=False,
        ),
        Index(
            "idx_imx_zkevm_sepolia_logs_block_hash_log_index",
            "block_hash",
            "log_index",
            unique=True,
        ),
        UniqueConstraint(
            "transaction_hash",
            "log_index",
            name="uq_imx_zkevm_sepolia_log_index_transaction_hash_log_index",
        ),
        PrimaryKeyConstraint(
            "transaction_hash", "log_index", name="pk_imx_zkevm_sepolia_log_index"
        ),
    )
    transaction_hash = Column(
        VARCHAR(256),
        ForeignKey("imx_zkevm_sepolia_transactions.hash", ondelete="CASCADE"),
        nullable=False,
        index=True,
    )


class ImxZkevmSepoliaReorgs(EvmBasedReorgs):
    __tablename__ = "imx_zkevm_sepolia_reorgs"


### ABI Jobs


class AbiJobs(Base):
    __tablename__ = "abi_jobs"

    __table_args__ = (
        UniqueConstraint(
            "chain",
            "address",
            "abi_selector",
            "customer_id",
            name="uq_abi_jobs",
        ),
    )

    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    address = Column(LargeBinary, nullable=False, index=True)
    user_id = Column(UUID(as_uuid=True), nullable=False, index=True)
    customer_id = Column(UUID(as_uuid=True), nullable=True, index=True)
    abi_selector = Column(VARCHAR(256), nullable=False, index=True)
    chain = Column(VARCHAR(256), nullable=False, index=True)
    abi_name = Column(VARCHAR(256), nullable=False, index=True)
    status = Column(VARCHAR(256), nullable=False, index=True)
    historical_crawl_status = Column(VARCHAR(256), nullable=False, index=True)
    progress = Column(Integer, nullable=False, index=False)
    moonworm_task_pickedup = Column(Boolean, nullable=False, index=False)
    abi = Column(Text, nullable=False)
    created_at = Column(
        DateTime(timezone=True), server_default=utcnow(), nullable=False
    )
    updated_at = Column(
        DateTime(timezone=True), server_default=utcnow(), nullable=False
    )


class AbiSubscriptions(Base):
    __tablename__ = "abi_subscriptions"

    __table_args__ = (PrimaryKeyConstraint("abi_job_id", "subscription_id"),)

    abi_job_id = Column(
        UUID(as_uuid=True),
        ForeignKey("abi_jobs.id"),
        nullable=False,
        index=True,
    )
    subscription_id = Column(UUID(as_uuid=True), nullable=False, index=True)
    created_at = Column(
        DateTime(timezone=True), server_default=utcnow(), nullable=False
    )



"""init v3 index

Revision ID: 0ea24314c181
Revises: 
Create Date: 2024-05-23 17:39:53.824404

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '0ea24314c181'
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('abi_jobs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('address', sa.LargeBinary(), nullable=False),
    sa.Column('user_id', sa.UUID(), nullable=False),
    sa.Column('customer_id', sa.UUID(), nullable=True),
    sa.Column('abi_selector', sa.VARCHAR(length=256), nullable=False),
    sa.Column('chain', sa.VARCHAR(length=256), nullable=False),
    sa.Column('abi_name', sa.VARCHAR(length=256), nullable=False),
    sa.Column('status', sa.VARCHAR(length=256), nullable=False),
    sa.Column('historical_crawl_status', sa.VARCHAR(length=256), nullable=False),
    sa.Column('progress', sa.Integer(), nullable=False),
    sa.Column('moonworm_task_pickedup', sa.Boolean(), nullable=False),
    sa.Column('abi', sa.Text(), nullable=False),
    sa.Column('created_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.Column('updated_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_abi_jobs')),
    sa.UniqueConstraint('chain', 'address', 'abi_selector', 'customer_id', name='uq_abi_jobs')
    )
    op.create_index(op.f('ix_abi_jobs_abi_name'), 'abi_jobs', ['abi_name'], unique=False)
    op.create_index(op.f('ix_abi_jobs_abi_selector'), 'abi_jobs', ['abi_selector'], unique=False)
    op.create_index(op.f('ix_abi_jobs_address'), 'abi_jobs', ['address'], unique=False)
    op.create_index(op.f('ix_abi_jobs_chain'), 'abi_jobs', ['chain'], unique=False)
    op.create_index(op.f('ix_abi_jobs_customer_id'), 'abi_jobs', ['customer_id'], unique=False)
    op.create_index(op.f('ix_abi_jobs_historical_crawl_status'), 'abi_jobs', ['historical_crawl_status'], unique=False)
    op.create_index(op.f('ix_abi_jobs_status'), 'abi_jobs', ['status'], unique=False)
    op.create_index(op.f('ix_abi_jobs_user_id'), 'abi_jobs', ['user_id'], unique=False)
    op.create_table('ethereum_blocks',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_ethereum_blocks'))
    )
    op.create_index(op.f('ix_ethereum_blocks_block_number'), 'ethereum_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_ethereum_blocks_block_timestamp'), 'ethereum_blocks', ['block_timestamp'], unique=False)
    op.create_table('ethereum_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_ethereum_reorgs'))
    )
    op.create_index(op.f('ix_ethereum_reorgs_block_hash'), 'ethereum_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_ethereum_reorgs_block_number'), 'ethereum_reorgs', ['block_number'], unique=False)
    op.create_table('polygon_blocks',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_timestamp', sa.BigInteger(), nullable=False),
    sa.Column('parent_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.PrimaryKeyConstraint('block_number', name=op.f('pk_polygon_blocks'))
    )
    op.create_index(op.f('ix_polygon_blocks_block_number'), 'polygon_blocks', ['block_number'], unique=False)
    op.create_index(op.f('ix_polygon_blocks_block_timestamp'), 'polygon_blocks', ['block_timestamp'], unique=False)
    op.create_table('polygon_reorgs',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.PrimaryKeyConstraint('id', name=op.f('pk_polygon_reorgs'))
    )
    op.create_index(op.f('ix_polygon_reorgs_block_hash'), 'polygon_reorgs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_polygon_reorgs_block_number'), 'polygon_reorgs', ['block_number'], unique=False)
    op.create_table('ethereum_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['ethereum_blocks.block_number'], name=op.f('fk_ethereum_transactions_block_number_ethereum_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_ethereum_transactions'))
    )
    op.create_index(op.f('ix_ethereum_transactions_block_hash'), 'ethereum_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_ethereum_transactions_block_number'), 'ethereum_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_ethereum_transactions_from_address'), 'ethereum_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_ethereum_transactions_hash'), 'ethereum_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_ethereum_transactions_index'), 'ethereum_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_ethereum_transactions_selector'), 'ethereum_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_ethereum_transactions_to_address'), 'ethereum_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_ethereum_transactions_type'), 'ethereum_transactions', ['type'], unique=False)
    op.create_table('polygon_transactions',
    sa.Column('block_number', sa.BigInteger(), nullable=False),
    sa.Column('hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('from_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('to_address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('type', sa.Integer(), nullable=True),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['block_number'], ['polygon_blocks.block_number'], name=op.f('fk_polygon_transactions_block_number_polygon_blocks'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('hash', name=op.f('pk_polygon_transactions'))
    )
    op.create_index(op.f('ix_polygon_transactions_block_hash'), 'polygon_transactions', ['block_hash'], unique=False)
    op.create_index(op.f('ix_polygon_transactions_block_number'), 'polygon_transactions', ['block_number'], unique=False)
    op.create_index(op.f('ix_polygon_transactions_from_address'), 'polygon_transactions', ['from_address'], unique=False)
    op.create_index(op.f('ix_polygon_transactions_hash'), 'polygon_transactions', ['hash'], unique=True)
    op.create_index(op.f('ix_polygon_transactions_index'), 'polygon_transactions', ['index'], unique=False)
    op.create_index(op.f('ix_polygon_transactions_selector'), 'polygon_transactions', ['selector'], unique=False)
    op.create_index(op.f('ix_polygon_transactions_to_address'), 'polygon_transactions', ['to_address'], unique=False)
    op.create_index(op.f('ix_polygon_transactions_type'), 'polygon_transactions', ['type'], unique=False)
    op.create_table('ethereum_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['ethereum_transactions.hash'], name=op.f('fk_ethereum_logs_transaction_hash_ethereum_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_ethereum_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_ethereum_log_index_transaction_hash_log_index')
    )
    op.create_index('idx_ethereum_logs_block_hash_log_index', 'ethereum_logs', ['block_hash', 'log_index'], unique=True)
    op.create_index(op.f('ix_ethereum_logs_address'), 'ethereum_logs', ['address'], unique=False)
    op.create_index(op.f('ix_ethereum_logs_block_hash'), 'ethereum_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_ethereum_logs_transaction_hash'), 'ethereum_logs', ['transaction_hash'], unique=False)
    op.create_table('polygon_logs',
    sa.Column('transaction_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('block_hash', sa.VARCHAR(length=256), nullable=False),
    sa.Column('address', sa.LargeBinary(length=20), nullable=False),
    sa.Column('row_id', sa.BigInteger(), nullable=False),
    sa.Column('selector', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic1', sa.VARCHAR(length=256), nullable=True),
    sa.Column('topic2', sa.VARCHAR(length=256), nullable=True),
    sa.Column('log_index', sa.BigInteger(), nullable=False),
    sa.Column('path', sa.Text(), nullable=False),
    sa.Column('indexed_at', sa.DateTime(timezone=True), server_default=sa.text("TIMEZONE('utc', statement_timestamp())"), nullable=False),
    sa.ForeignKeyConstraint(['transaction_hash'], ['polygon_transactions.hash'], name=op.f('fk_polygon_logs_transaction_hash_polygon_transactions'), ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('transaction_hash', 'log_index', name='pk_polygon_log_index'),
    sa.UniqueConstraint('transaction_hash', 'log_index', name='uq_polygon_log_index_transaction_hash_log_index')
    )
    op.create_index(op.f('ix_polygon_logs_address'), 'polygon_logs', ['address'], unique=False)
    op.create_index(op.f('ix_polygon_logs_block_hash'), 'polygon_logs', ['block_hash'], unique=False)
    op.create_index(op.f('ix_polygon_logs_transaction_hash'), 'polygon_logs', ['transaction_hash'], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_polygon_logs_transaction_hash'), table_name='polygon_logs')
    op.drop_index(op.f('ix_polygon_logs_block_hash'), table_name='polygon_logs')
    op.drop_index(op.f('ix_polygon_logs_address'), table_name='polygon_logs')
    op.drop_table('polygon_logs')
    op.drop_index(op.f('ix_ethereum_logs_transaction_hash'), table_name='ethereum_logs')
    op.drop_index(op.f('ix_ethereum_logs_block_hash'), table_name='ethereum_logs')
    op.drop_index(op.f('ix_ethereum_logs_address'), table_name='ethereum_logs')
    op.drop_index('idx_ethereum_logs_block_hash_log_index', table_name='ethereum_logs')
    op.drop_table('ethereum_logs')
    op.drop_index(op.f('ix_polygon_transactions_type'), table_name='polygon_transactions')
    op.drop_index(op.f('ix_polygon_transactions_to_address'), table_name='polygon_transactions')
    op.drop_index(op.f('ix_polygon_transactions_selector'), table_name='polygon_transactions')
    op.drop_index(op.f('ix_polygon_transactions_index'), table_name='polygon_transactions')
    op.drop_index(op.f('ix_polygon_transactions_hash'), table_name='polygon_transactions')
    op.drop_index(op.f('ix_polygon_transactions_from_address'), table_name='polygon_transactions')
    op.drop_index(op.f('ix_polygon_transactions_block_number'), table_name='polygon_transactions')
    op.drop_index(op.f('ix_polygon_transactions_block_hash'), table_name='polygon_transactions')
    op.drop_table('polygon_transactions')
    op.drop_index(op.f('ix_ethereum_transactions_type'), table_name='ethereum_transactions')
    op.drop_index(op.f('ix_ethereum_transactions_to_address'), table_name='ethereum_transactions')
    op.drop_index(op.f('ix_ethereum_transactions_selector'), table_name='ethereum_transactions')
    op.drop_index(op.f('ix_ethereum_transactions_index'), table_name='ethereum_transactions')
    op.drop_index(op.f('ix_ethereum_transactions_hash'), table_name='ethereum_transactions')
    op.drop_index(op.f('ix_ethereum_transactions_from_address'), table_name='ethereum_transactions')
    op.drop_index(op.f('ix_ethereum_transactions_block_number'), table_name='ethereum_transactions')
    op.drop_index(op.f('ix_ethereum_transactions_block_hash'), table_name='ethereum_transactions')
    op.drop_table('ethereum_transactions')
    op.drop_index(op.f('ix_polygon_reorgs_block_number'), table_name='polygon_reorgs')
    op.drop_index(op.f('ix_polygon_reorgs_block_hash'), table_name='polygon_reorgs')
    op.drop_table('polygon_reorgs')
    op.drop_index(op.f('ix_polygon_blocks_block_timestamp'), table_name='polygon_blocks')
    op.drop_index(op.f('ix_polygon_blocks_block_number'), table_name='polygon_blocks')
    op.drop_table('polygon_blocks')
    op.drop_index(op.f('ix_ethereum_reorgs_block_number'), table_name='ethereum_reorgs')
    op.drop_index(op.f('ix_ethereum_reorgs_block_hash'), table_name='ethereum_reorgs')
    op.drop_table('ethereum_reorgs')
    op.drop_index(op.f('ix_ethereum_blocks_block_timestamp'), table_name='ethereum_blocks')
    op.drop_index(op.f('ix_ethereum_blocks_block_number'), table_name='ethereum_blocks')
    op.drop_table('ethereum_blocks')
    op.drop_index(op.f('ix_abi_jobs_user_id'), table_name='abi_jobs')
    op.drop_index(op.f('ix_abi_jobs_status'), table_name='abi_jobs')
    op.drop_index(op.f('ix_abi_jobs_historical_crawl_status'), table_name='abi_jobs')
    op.drop_index(op.f('ix_abi_jobs_customer_id'), table_name='abi_jobs')
    op.drop_index(op.f('ix_abi_jobs_chain'), table_name='abi_jobs')
    op.drop_index(op.f('ix_abi_jobs_address'), table_name='abi_jobs')
    op.drop_index(op.f('ix_abi_jobs_abi_selector'), table_name='abi_jobs')
    op.drop_index(op.f('ix_abi_jobs_abi_name'), table_name='abi_jobs')
    op.drop_table('abi_jobs')
    # ### end Alembic commands ###
